package cn.iocoder.yudao.module.system.api.robot;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.tenant.core.context.TenantContextHolder;
import cn.iocoder.yudao.module.system.api.robot.dto.RobotStatusDataErrorDTO;
import cn.iocoder.yudao.module.system.api.robot.dto.RobotStatusDataPoseDTO;
import cn.iocoder.yudao.module.system.api.robot.vo.RobotReactiveStatusDTO;
import cn.iocoder.yudao.module.system.constant.robot.RobotTaskChcheConstant;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotWarnCodeMappingDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotWarnMsgDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotWarnCodeMappingMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotWarnMsgMapper;
import cn.iocoder.yudao.module.system.enums.robot.RobotWarnType;
import cn.iocoder.yudao.module.system.service.robot.RobotInformationService;
import cn.iocoder.yudao.module.system.service.robot.RobotWarnMsgService;
import cn.iocoder.yudao.module.system.util.redis.RedisUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@RestController // 提供 RESTful API 接口，给 Feign 调用
@Validated
public class RobotReactiveStatusApiImpl implements RobotReactiveStatusApi {

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private RobotWarnMsgService warnMsgService;

    @Resource
    private RobotWarnMsgMapper warnMsgMapper;

    @Resource
    private RobotWarnCodeMappingMapper warnCodeMappingMapper;

    @Autowired
    private RobotInformationService robotInformationService;

    @Override
    public void robotReactiveStatus(String message) {
        TenantContextHolder.setTenantId(1L);
        RobotReactiveStatusDTO data = JSON.parseObject(message, RobotReactiveStatusDTO.class);
        String floorAreaKey = RobotTaskChcheConstant.ROBOT_FLOOR_AREA + data.getMac();
        redisUtil.set(floorAreaKey, JSON.toJSONString(data.getFloorZone()));

        //机器人异常等级
        if (ObjectUtil.isEmpty(data.getErrCode())) {
            String errorLevelKey = RobotTaskChcheConstant.ROBOT_ERROR_LEVEL + data.getMac();
            String errorMsgKey = RobotTaskChcheConstant.ROBOT_ERROR_MSG + data.getMac();
            redisUtil.del(errorLevelKey);
            redisUtil.del(errorMsgKey);
        }else {
            addRobotErrorMsg(data);
        }
    }

    /**
     * 添加异常
     * @param data
     */
    private void addRobotErrorMsg(RobotReactiveStatusDTO data) {
        String robotNo = robotInformationService.getRobotNoByMac(data.getMac());
        List<RobotStatusDataErrorDTO> errCode = data.getErrCode();
        List<String> warnCodes =
                errCode.stream().map(RobotStatusDataErrorDTO::getErrorCode).collect(Collectors.toList());
        List<RobotWarnCodeMappingDO> robotWarnCodeMappingDOS =
                warnCodeMappingMapper.selectList(new LambdaQueryWrapper<RobotWarnCodeMappingDO>()
                        .in(RobotWarnCodeMappingDO::getWarnCode, warnCodes));
        if (ObjectUtil.isEmpty(robotWarnCodeMappingDOS)) {
            log.info("查不对应编号的告警信息 :{}", JSON.toJSONString(warnCodes));
            return;
        }

        Map<String, List<RobotWarnCodeMappingDO>> warnCodeMapping =
                robotWarnCodeMappingDOS.stream().collect(Collectors.groupingBy(RobotWarnCodeMappingDO::getWarnCode));

        List<RobotWarnMsgDO> warnMsgDOS = new ArrayList<>();

        //机器人异常等级
        String errorLevelKey = RobotTaskChcheConstant.ROBOT_ERROR_LEVEL + data.getMac();
        Object errorLevel = redisUtil.get(errorLevelKey);

        String errorMsgKey = RobotTaskChcheConstant.ROBOT_ERROR_MSG + data.getMac();
        Object errorMsg = redisUtil.get(errorMsgKey);

        Integer level = ObjectUtil.isEmpty(errorLevel) ? 0 : Integer.valueOf(errorLevel.toString());

        String msg = "";
        int i = 0;
        for (RobotStatusDataErrorDTO robotStatusData : errCode) {
            List<RobotWarnCodeMappingDO> mappingDOS = warnCodeMapping.get(robotStatusData.getErrorCode());
            if (ObjectUtil.isEmpty(mappingDOS)) {
                log.info("当前告警类型查不到对应的告警信息 :{}", robotStatusData.getErrorCode());
                continue;
            }
            RobotWarnMsgDO warnMsg = RobotWarnMsgDO.builder().warnLevel(Integer.valueOf(robotStatusData.getCodeLevel()))
                    .warnCode(robotStatusData.getErrorCode())
                    .robotNo(robotNo)
                    .warnType(RobotWarnType.ROBOT_WARN.getType())
                    .warnMsg(robotNo+"_"+mappingDOS.get(0).getWarnMsg())
                    .warnSolve(mappingDOS.get(0).getWarnSolve())
                    .build();
            warnMsgDOS.add(warnMsg);

            if (level.intValue() < Integer.valueOf(robotStatusData.getCodeLevel()).intValue()) {
                level = Integer.valueOf(robotStatusData.getCodeLevel());
                errorMsg = warnMsg.getWarnMsg();
            }
            if (i< Integer.valueOf(robotStatusData.getCodeLevel()).intValue()) {
                i = Integer.valueOf(robotStatusData.getCodeLevel());
                msg = warnMsg.getWarnMsg();
            }
        }
        redisUtil.set(errorLevelKey, level);
        redisUtil.set(errorMsgKey, errorMsg);

        warnMsgService.sendWarnMsgToWebsocket(msg);

        warnMsgMapper.insertBatch(warnMsgDOS);
    }
}
