package cn.iocoder.yudao.module.system.controller.admin.robot;

import cn.iocoder.yudao.module.system.controller.admin.robot.proceed.*;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskProceedDO;
import cn.iocoder.yudao.module.system.service.robot.proceed.RobotTaskProceedService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.*;


@Tag(name = "管理后台 - 车辆衔接任务")
@RestController
@RequestMapping("/robot/task-proceed")
@Validated
public class RobotTaskProceedController {

    @Resource
    private RobotTaskProceedService taskProceedService;

    @PostMapping("/create")
    @Operation(summary = "创建车辆衔接任务")
    @PreAuthorize("@ss.hasPermission('robot:task-proceed:create')")
    public CommonResult<Long> createTaskProceed(@Valid @RequestBody RobotTaskProceedSaveReqVO createReqVO) {
        return success(taskProceedService.createTaskProceed(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新车辆衔接任务")
    @PreAuthorize("@ss.hasPermission('robot:task-proceed:update')")
    public CommonResult<Boolean> updateTaskProceed(@Valid @RequestBody RobotTaskProceedSaveReqVO updateReqVO) {
        taskProceedService.updateTaskProceed(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除车辆衔接任务")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('robot:task-proceed:delete')")
    public CommonResult<Boolean> deleteTaskProceed(@RequestParam("id") Long id) {
        taskProceedService.deleteTaskProceed(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得车辆衔接任务")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('robot:task-proceed:query')")
    public CommonResult<RobotTaskProceedRespVO> getTaskProceed(@RequestParam("id") Long id) {
        RobotTaskProceedDO taskProceed = taskProceedService.getTaskProceed(id);
        return success(BeanUtils.toBean(taskProceed, RobotTaskProceedRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得车辆衔接任务分页")
    @PreAuthorize("@ss.hasPermission('robot:task-proceed:query')")
    public CommonResult<PageResult<RobotTaskProceedRespVO>> getTaskProceedPage(@Valid RobotTaskProceedPageReqVO pageReqVO) {
        PageResult<RobotTaskProceedDO> pageResult = taskProceedService.getTaskProceedPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, RobotTaskProceedRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出车辆衔接任务 Excel")
    @PreAuthorize("@ss.hasPermission('robot:task-proceed:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportTaskProceedExcel(@Valid RobotTaskProceedPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<RobotTaskProceedDO> list = taskProceedService.getTaskProceedPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "车辆衔接任务.xls", "数据", RobotTaskProceedRespVO.class,
                        BeanUtils.toBean(list, RobotTaskProceedRespVO.class));
    }

}