package cn.iocoder.yudao.module.system.service.robot.proceed;

import cn.iocoder.yudao.module.system.controller.admin.robot.proceed.*;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskProceedDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskProceedMapper;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.TASK_PROCEED_NOT_EXISTS;

/**
 * 车辆衔接任务 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class RobotTaskProceedServiceImpl implements RobotTaskProceedService {

    @Resource
    private RobotTaskProceedMapper taskProceedMapper;

    @Override
    public Long createTaskProceed(RobotTaskProceedSaveReqVO createReqVO) {
        // 插入
        RobotTaskProceedDO taskProceed = BeanUtils.toBean(createReqVO, RobotTaskProceedDO.class);
        taskProceedMapper.insert(taskProceed);
        // 返回
        return taskProceed.getId();
    }

    @Override
    public void updateTaskProceed(RobotTaskProceedSaveReqVO updateReqVO) {
        // 校验存在
        validateTaskProceedExists(updateReqVO.getId());
        // 更新
        RobotTaskProceedDO updateObj = BeanUtils.toBean(updateReqVO, RobotTaskProceedDO.class);
        taskProceedMapper.updateById(updateObj);
    }

    @Override
    public void deleteTaskProceed(Long id) {
        // 校验存在
        validateTaskProceedExists(id);
        // 删除
        taskProceedMapper.deleteById(id);
    }

    private void validateTaskProceedExists(Long id) {
        if (taskProceedMapper.selectById(id) == null) {
            throw exception(TASK_PROCEED_NOT_EXISTS);
        }
    }

    @Override
    public RobotTaskProceedDO getTaskProceed(Long id) {
        return taskProceedMapper.selectById(id);
    }

    @Override
    public PageResult<RobotTaskProceedDO> getTaskProceedPage(RobotTaskProceedPageReqVO pageReqVO) {
        return taskProceedMapper.selectPage(pageReqVO);
    }

}