package cn.iocoder.yudao.module.system.service.tool;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.module.mqtt.api.common.CommonApi;
import cn.iocoder.yudao.module.mqtt.api.task.dto.RobotAcceptTaskDTO;
import cn.iocoder.yudao.module.mqtt.enums.task.ExecutionTypeEnum;
import cn.iocoder.yudao.module.system.api.path.vo.RobotClosePathPlantingDTO;
import cn.iocoder.yudao.module.system.constant.path.PathPlanningTopicConstant;
import cn.iocoder.yudao.module.system.constant.robot.RobotTopicConstant;
import cn.iocoder.yudao.module.system.controller.admin.config.dto.TaskOrderConfigDTO;
import cn.iocoder.yudao.module.system.controller.admin.config.vo.CommonConfigVO;
import cn.iocoder.yudao.module.system.controller.admin.tool.dto.CleanAgvDTO;
import cn.iocoder.yudao.module.system.dal.dataobject.config.CommonConfigDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotWarnMsgDO;
import cn.iocoder.yudao.module.system.dal.mysql.config.CommonConfigMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotWarnMsgMapper;
import cn.iocoder.yudao.module.system.enums.config.CommandConfigTypeEnum;
import cn.iocoder.yudao.module.system.enums.robot.RobotWarnType;
import cn.iocoder.yudao.module.system.service.robot.RobotInformationService;
import cn.iocoder.yudao.module.system.service.robot.RobotWarnMsgService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

@Slf4j
@Service
@Validated
public class ToolsServiceImpl implements ToolsService {

    @Resource
    private CommonApi commonApi;

    @Resource
    private CommonConfigMapper configMapper;

    @Resource
    private RobotWarnMsgService warnMsgService;

    @Resource
    private RobotWarnMsgMapper warnMsgMapper;

    @Resource
    private RobotInformationService informationService;

    @Autowired
    private RobotInformationService robotInformationService;

    /**
     * 发送时间、优先级、距离的权重给PP
     *
     * @return
     */
    @Override
    public String sendPPsortConfig() {

        CommonConfigDO commonConfigDO = configMapper.selectOne(new LambdaQueryWrapper<CommonConfigDO>()
                .eq(CommonConfigDO::getConfigType, CommandConfigTypeEnum.CHARG_CONFIG.getType()));

        if (ObjectUtil.isEmpty(commonConfigDO) || ObjectUtil.isEmpty(commonConfigDO.getConfigStr())) {
            log.info("没有配置路径规划权重");
            addWarnMsg();
            return "没有配置路径规划权重";
        }

        CommonConfigVO chargeConfig = JSONUtil.toBean(commonConfigDO.getConfigStr(), CommonConfigVO.class);
        if (ObjectUtil.isEmpty(chargeConfig.getPathSortConfig())) {
            addWarnMsg();
            return "没有配置路径规划权重";
        }

        CommonConfigDO config = configMapper.selectOne(new LambdaQueryWrapper<CommonConfigDO>()
                .eq(CommonConfigDO::getConfigType, chargeConfig.getPathSortConfig()));
        if (ObjectUtil.isEmpty(config)) {
            addWarnMsg();
            return "没有配置路径规划权重";
        }

        TaskOrderConfigDTO taskOrderConfigDTO = JSONUtil.toBean(config.getConfigStr(), TaskOrderConfigDTO.class);
        commonApi.commonMethod(taskOrderConfigDTO,
                PathPlanningTopicConstant.SEND_SORT_CONFIG_TO_PP);

        return "发送数据库权重配置成功";
    }

    /**
     * 清除交管
     *
     * @return
     */
    @Override
    public String cleanAgv(String robotNo) {
        CleanAgvDTO build = CleanAgvDTO.builder().robotNo(robotNo).build();
        commonApi.commonMethod(build, PathPlanningTopicConstant.CLEAN_AGV);
        return "清除交管成功 " + robotNo;
    }

    /**
     * 关闭任务
     * @param taskDetailId
     * @return
     */
    @Override
    public String closeTaskDetail(String taskDetailId) {
        RobotInformationDO informationDO = informationService.getRobotInformationByDetailId(taskDetailId);
        RobotClosePathPlantingDTO closePathPlanting = RobotClosePathPlantingDTO.builder()
                .robotNo(informationDO.getRobotNo())
                .id(taskDetailId)
                .build();
        commonApi.commonMethod(closePathPlanting, PathPlanningTopicConstant.KILL_TASK);

        RobotAcceptTaskDTO robotTask = new RobotAcceptTaskDTO();
        robotTask.setOrderId(taskDetailId);
        String mac = robotInformationService.getMacByRobotNo(informationDO.getRobotNo());
        robotTask.setTopic(RobotTopicConstant.ROBOT_TASK_MOVE_TOPIC + mac);
        robotTask.setExecutionType(ExecutionTypeEnum.CANCEL.getType());
        commonApi.commonMethod(robotTask, robotTask.getTopic());

        return "关闭任务成功";
    }

    @Override
    public String updateWarnCode() {
        List<RobotInformationDO> robots = robotInformationService.getAllRobot();
        if (ObjectUtil.isEmpty(robots)){
            return "暂无机器人";
        }

        for (RobotInformationDO robot : robots) {
            commonApi.commonMethodStr("请同步告警码值", RobotTopicConstant.UPDATE_ERROR_TOPIC+robot.getMacAddress());
            try {
                //怕车机同时间同步对内存有影响
                Thread.sleep(500);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        return "同步完成";
    }


    public void addWarnMsg() {
        RobotWarnMsgDO warnMsg = RobotWarnMsgDO.builder().warnLevel(4)
                .warnCode("NOT_PP_CONFIG")
                .robotNo("PP")
                .warnType(RobotWarnType.ROBOT_WARN.getType())
                .warnMsg("没有配置路径规划权重")
                .warnSolve("配置页面设置")
                .build();
        warnMsgService.sendWarnMsgToWebsocket(warnMsg.getWarnMsg());
        warnMsgMapper.insertBatch(Arrays.asList(warnMsg));
    }


}
